function [y,res]=chowlin(grocer_namey,varargin)
 
// PURPOSE: Temporal disaggregation using the Chow-Lin method
// (high level function with vectors, matrices or ts and
// the possibility of default parmaters)
// ------------------------------------------------------------
// INPUT:
// * grocer_namey = a time series, a real (nx1) vector or a
//   string equal to the name of a time series or a (nx1) real
//   vector between quote, representing the low frequency data
//   that must be desaggregated
// * varargin = arguments which can be:
//   . a time series
//   . a real (nx1) vector
//   . a string equal to the name of a time series or a (nx1)
//     real vector between quotes
//   . a list of such objects
//   . the string 'divfq=n' where n is the number of high
//    frequency data points for each low frequency data points
//   . the string 'typemin=xxx' where xxx is the maximisation
//     method (llike -default- or wls)
//   . the string 'ta=n' where n is the aggregation type:
//      n=-1 (default) ---> sum (flow)
//      n=0 ---> average (index)
//      n=i ---> i th element (stock) ---> interpolation
//   . the string 'delta=x' where x is the increment used for
//     numerical derivation
// ------------------------------------------------------------
// OUTPUT:
// * y = the disaggregated variable
// * res = a results tlist with:
//   - res('meth')      = 'Chow-Lin'
//   - res('ta')        = type of disaggregation
//   - res('nobs_lf')   = nobs. of low frequency data
//   - res('nobs_hf')   = nobs. of high-frequency data
//   - res('pred')      = number of extrapolations
//   - res('s')         = frequency conversion between low and high freq.
//   - res('p')         = number of regressors (including intercept)
//   - res('y_lf')      = low frequency data
//   - res('indicator') = high frequency indicators
//   - res('y')         = high frequency estimate
//   - res('y_dt')      = high frequency estimate: standard deviation
//   - res('y_up')      = high frequency estimate: sd + sigma
//   - res('y_lo')      = high frequency estimate: sd - sigma
//   - res('resid_hf')  = high frequency residuals
//   - res('resid_lf)   = low frequency residuals
//   - res('beta')      = estimated model parameters
//   - res('sd')        = estimated model parameters: standard deviation
//   - res('tstat')     = estimated model parameters: t ratios
//   - res('pvalue')    = pvalue of the betas
//   - res('rho')       = innovational parameter
//   - res('aic')       = Information criterion: AIC
//   - res('bic')       = Information criterion: BIC
//   - res('typemin')   = method of estimation
//   - res('llike')     = Log-likelihood at the estimated parameters
//   - res('sigma')     = Variance at the estimated parameters
//   - res('namey')     = Name of the high frequency aggregate
//   - res('namex')     = Name of the low frequency indicators
//   - res('prests')    = a boolean indicating the presence or
//     absence of a time series in the regression
//   - res('bounds')    = if there is a timeseries in the
//     regression, the bounds of the regression
// ------------------------------------------------------------
// REFERENCE: Chow, G. & Lin, A.L. (1971) "Best linear unbiased
// distribution and extrapolation of economic time series by related
// series", Review of Economic and Statistics, vol. 53, n. 4, p. 372-375.
// ------------------------------------------------------------
// Copyright: Eric Dubois 2005
// translated and adpated to Scilab from Matlab programs written
// by:
// Enrique M. Quilis
// Instituto Nacional de Estadistica
// Paseo de la Castellana, 183
// 28046 - Madrid (SPAIN)
 
// set defaults
grocer_calc_divfq=%t
grocer_delta=sqrt(%eps)
grocer_typemin='llike'
grocer_ta=-1
grocer_prt=%t
 
grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   if typeof(varargin(grocer_i)) == 'string' then
      varaux=strsubst(varargin(grocer_i),' ','')
      str6=part(varargin(grocer_i),[1:6])
      str8=part(varargin(grocer_i),[1:8])
      str3=part(varargin(grocer_i),[1:3])
      if str6 == 'divfq=' then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null()
         grocer_calc_divfq=%f
      elseif str6 == 'delta=' then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null()
      elseif str8 == 'typemin=' then
         grocer_typemin=part(varaux,9:length(varaux))
         varargin(grocer_i)=null()
      elseif str3 == 'ta=' then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null()
      elseif varaux == 'noprint' then
         grocer_prt=%f
         varargin(grocer_i)=null()
      end
   elseif typeof(varargin(grocer_i)) ~= 'constant' &...
   typeof(varargin(grocer_i)) ~= 'ts' then
      error('wrong type for entry number '+string(grocer_i+1))
   end
end
 
if grocer_calc_divfq then
   [Y,x,grocer_namendo,grocer_namexos,grocer_prestshf,grocer_boundshf,grocer_divfq]=explo_agreg(grocer_ta,grocer_namey,varargin(:))
else
   [Y,x,grocer_namendo,grocer_namexos,grocer_prestshf,grocer_boundshf,tmp_divfq]=explo_agreg(grocer_ta,grocer_namey,varargin(:))
   if grocer_divfq ~= tmp_divfq then
    warning('frequency conversion divfq ('+string(grocer_divfq)+') doesn''t macth: it has been replaced by '+string(tmp_divfq))
    grocer_divfq = tmp_divfq
   end
end
 
[y,res]=chowlin1(Y,x,grocer_ta,grocer_divfq,grocer_delta,grocer_typemin)
 
res(1)($+1)='namey'
res('namey')=grocer_namendo
res(1)($+1)='namex'
res('namex')=grocer_namexos
res(1)($+1) = 'prests'
res('prests') = grocer_prestshf
 
if grocer_prestshf then
   res(1)($+1) = 'bounds'
   res('bounds') = grocer_boundshf
   y=reshape(y,grocer_boundshf(1))
   res('y_hf')=y
   res('y_dt')=reshape(res('y_dt'),grocer_boundshf(1))
   res('y_up')=reshape(res('y_up'),grocer_boundshf(1))
   res('y_lo')=reshape(res('y_lo'),grocer_boundshf(1))
end
 
if grocer_prt then
   prtdisag(res,%io(2))
end
endfunction
