function [rsur]=suruneq(grocer_namecoef,grocer_speccara,grocer_speccarb,grocer_crit,grocer_dropna,varargin)
 
// PURPOSE: provide Zellner Seemingly Unrelated Regression in
// the standard case when all equations are not estimated on the
// same period (subroutine of function sur)
// ------------------------------------------------------------
// INPUT:
// * grocer_namecoef = the names of the coefficients
// * grocer_speccara = column vector of characters that must be
//   found after the name of a coefficient
// * grocer_speccarb = column vector of characters that must be
//   found before the name of a coefficient
// * grocer_crit = a scalar, the convergence criterion
//   (optional; default =1e-4)
// * grocer_drpna = 'dropna' if the user wants to remove the
//     NA values from the data
// * varargin = equations written
// 'vary=coef1*varx1+...+coefi*varxi'
//   where:
//   - coefi = the name of a coefficient
//   - varxi = the name of a variable
// ------------------------------------------------------------
// OUTPUT:
// rsur=a tlist with
// - rsur('meth') = 'sur'
// - rsur('nobs') = # of observations
// - rsur('neqs') = # of estimated equations
// - rsur('ncoef') = # of estimated coefficients
// - rsur('beta') = bhat
// - rsur('tstat') = t-stats
// - rsur('pvalue') = pvalue of the betas
// - rsur('sigma') = covariance matrix of the residuals
// - rsur('sigu') = (1 x neqs) sum of squared residuals
// - rsur('sigu') = (1 x neqs) sum of squared residuals
// - rsur('dw') = (1 x neqs) Durbin-Watson
// - rsur('prests') = boolean indicating the presence or
//     absence of a time series in the regression
// - rsur('namecoef') = (ncoef x 1) mame of the coeffcients
// - rsur('namey') = name of endogenous variables
// - rsur('eqs') = list of the neqs equations
// - rsur('coefs') = list of the coefs names in each equation
// - rsur('bounds') = list of the bounds in each equation
// - rsur('dropna') = boolean indicating if NAs had
//		    been dropped
// - rsur('nonna') = vector indicating position of
//     non-NA values (if the option 'dropna' was active)
// ------------------------------------------------------------
// Copyright: Eric Dubois 2006-2007
// http://dubois.ensae.net/grocer.html
 
 
grocer_diffd=diff_date(grocer_boundsvar($),grocer_boundsvar(1))+1
 
// explode the equation into the x,y matrices, the name of the
// endogenous variables, the list of indexes of the coefficients
// in each equation and the bounds and the boolean indicating
// the presence of a ts
[x,y,grocer_boundsvarb,grocer_prests,listcoef,namey,ncoefeqs,X]=...
explosys(grocer_namecoef,grocer_speccara,grocer_speccarb,%t,grocer_dropna,varargin(:))
 
fq=date2fq(grocer_boundsvar(1))
 
convg=grocer_crit*2
// uses results from ols as starting values
 
beta0=ols0(y,x)
resid=y-x*beta0
nbit=0
nobs=ones(grocer_neqs,1)
for i=1:grocer_neqs
   xi=X(:,i)
   nobs(i)=size(find(~isnan(xi)),2)
end
nobst=sum(nobs)
nobscum=[0 ; cumsum(nobs)]
ncoef=size(grocer_namecoef,1)
sigma=zeros(grocer_neqs,grocer_neqs)
omega=zeros(nobst,nobst)
 
// create the list of non na observations for all equations
X_nonna=list()
for i=1:grocer_neqs
   X_nonna($+1)=~isnan(X(:,i))
end
 
// create the list of common observations for any 2 equations
common_per=list()
for i=1:grocer_neqs
   for j=i:grocer_neqs
      common_per($+1)=X_nonna(i) & X_nonna(j)
   end
end
 
// create the list of common observations for any 2 equations
// by reference to the non na observations in each equation
common_nonna=list()
for i=1:grocer_neqs
   ai=X_nonna(i)
   fai=find(ai)
   for j=1:grocer_neqs
      aj=X_nonna(j)
      faiaj=find(ai & aj)
      ind=zeros(size(faiaj,2),1)
      for k=1:size(faiaj,2)
         ind(k)=find(fai == faiaj(k))
      end
      common_nonna($+1)=ind
   end
end
 
while (convg > grocer_crit & nbit < grocer_itmax) then
   nbit=nbit+1
   for i=1:grocer_neqs
      X(X_nonna(i),i)=resid(nobscum(i)+1:nobscum(i+1))
   end
 
   ind_per=0
   for i=1:grocer_neqs
      for j=i:grocer_neqs
         ind_per=ind_per+1
         omega_i_j=X(common_per(ind_per),i)'*X(common_per(ind_per),j)/sqrt(nobs(i)-ncoefeqs(i))...
                  /sqrt(nobs(j)-ncoefeqs(j))
         omega(nobscum(i)+common_nonna((i-1)*grocer_neqs+j),nobscum(j)+common_nonna((j-1)*grocer_neqs+i))=...
            omega_i_j*eye(size(common_nonna((i-1)*grocer_neqs+j),1),size(common_nonna((j-1)*grocer_neqs+i),1))
         sigma(i,j)=omega_i_j
      end
   end
   omega=omega+triu(omega,1)'
   [u,d,v]=svd(omega)
   sqrtomega=u* (diag(ones(size(omega,2),1) ./ sqrt(diag(d)))) *v'
   sqrtomegax=sqrtomega*x
   [bhat,ixpomegam1x]=ols0(sqrtomega*y,sqrtomegax)
 
   resid=y-x*bhat
   convg=sum(abs(bhat-beta0))
   beta0=bhat
end
 
residl=list()
for i=1:grocer_neqs
   residl($+1)=resid(nobscum(i)+1:nobscum(i+1))
end
 
tstat=bhat ./ sqrt(diag(ixpomegam1x))
pvalue=zeros(ncoef,1)
ds=diag(sigma)
ser=sqrt(ds)
sigu=ds .* (nobs-ncoefeqs)
corm=var2cor(sigma)
 
for i=1:ncoef
   pvalue(i)=(1-cdfnor("PQ",abs(tstat(i)),0,1))*2
end
 
dw=zeros(1,grocer_neqs)
listbounds=list()
for i=1:grocer_neqs
   xi=X(:,i)
 
   bd=[]
   if ~isnan(xi(1)) then
      bd=1
   end
   bd1=find(~isnan(xi(2:$)) & isnan(xi(1:$-1)))
   if ~isempty(bd1) then
      bd=[bd bd1+1]
   end
 
   bf=[]
   if ~isnan(xi($)) then
      bf=grocer_diffd
   end
   bf1=find(isnan(xi(2:$)) & ~isnan(xi(1:$-1)))
   if ~isempty(bf1) then
      bf=[bf1 bf]
   end
   b=matrix([bd ; bf],-1,1)+date2num(grocer_boundsvar(1))-1
   dates=num2date(b,fq)
   listbounds($+1)=dates
 
   // durbin-watson
   ediff = resid(nobscum(i)+2:nobscum(i+1))-resid(nobscum(i)+1:nobscum(i+1)-1)
   dw(i) = ediff'*ediff/sigu(i)
 
end
 
 
rsur=tlist(['results';'meth';'nobs';'neqs';'ncoef';'beta';...
'tstat';'pvalue';'resid';'sigma';'corr';'sigu';'ser';'dw';...
'prests';'namecoef';'namey';'eqs';'coefs';'bounds'],...
'sur',nobs,grocer_neqs,ncoefeqs,bhat,tstat,pvalue,...
residl,sigma,corm,sigu,ser,dw,grocer_prests,grocer_namecoef,...
namey,varargin,listcoef,listbounds)
 
if grocer_dropna then
   rsur(1)($+1)='nonna'
   rsur('nonna')=nonna
end
 
endfunction
