function [result]=pow_min(grocer_func,grocer_pin,varargin)
 
// PURPOSE: Powell minimization routine to minimize func
// ------------------------------------------------------------
// INPUT:
// * grocer_func = (minus likelihood) function to minimize
// * grocer_pin = parameter vector fed to func
// * varargin = 'maxit=x' to set the maximum # of iterations
//              (default = 500)
//
// ------------------------------------------------------------
// OUTPUT:
// result = a result list with:
//   - result('meth') = 'pow'
//   - result('b')    = (kx1) minimizing vector
//   - result('f')    = value of func at solution values
//   - result('hess') = hessian at solution values
//   - result('iter') = # number of iterations
//   - result('flag') = 0 for convergence, 1 for non-convergence
// ------------------------------------------------------------
// NOTE:
// grocer_func must take the form grocer_func(b,varargin)
// where:  b        = parameter vector (k x 1)
//         varargin = arguments passed to the function
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002
// adapted from:
// James P. LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jlesage@spatial-econometrics.com
 
// default options
grocer_tol = sqrt(%eps);
grocer_maxit = 500;
grocer_pflag = 0;
grocer_grad='numz0'
grocer_delta=1e-5
 
grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   if typeof(varargin(grocer_i)) == 'string' then
      if (part(varargin(grocer_i),[1:5]) == 'maxit' ...
      | part(varargin(grocer_i),[1:5]) == 'delta' ...
      | part(varargin(grocer_i),[1:3]) == 'tol' ...
      | part(varargin(grocer_i),[1:6]) == 'optprt') ...
      then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i) = null()
      else
         if part(varargin(grocer_i),[1:4]) == 'grad' then
            grocer_ch=strsubst('grocer_'+varargin(grocer_i),...
            '=','=''')+''''
            execstr(grocer_ch)
            varargin(grocer_i)=null()
         end
      end
   end
end
 
if grocer_grad == 'numz0' then
   grocer_chg='numz0(grocer_func,grocer_pin,grocer_n,...
   grocer_in,grocer_delta,varargin(:))'
else
   grocer_chg=grocer_grad+'(grocer_pin)'
end
 
grocer_epss = sqrt(%eps);
 
grocer_n = size(grocer_pin,1);
grocer_in=ones(grocer_n,1)
grocer_fret = grocer_func(grocer_pin,varargin(:))
grocer_pt = grocer_pin;
grocer_xi = eye(grocer_n,grocer_n);
 
grocer_iter = 0;
grocer_t = 1;
 
while grocer_iter<=grocer_maxit then
  grocer_iter = grocer_iter+1;
  grocer_fp = grocer_fret;
  grocer_ibig = 0;
  grocer_del = 0;
  grocer_i = 1;
 
  if grocer_pflag==1 then
     write(%io(2),'iteration # '+string(grocer_iter))
     write(%io(2),'value of function: '+string(grocer_fp))
     disp('coefficient values')
     disp(grocer_pin)
  end
 
  while grocer_i<=grocer_n then
    grocer_xit = grocer_xi(:,grocer_i);
    [grocer_pin grocer_fret] = linmin(grocer_pin,grocer_xit,grocer_tol,...
        grocer_func,varargin(:))
 
    if abs(grocer_fp-grocer_fret)>grocer_del then
       grocer_del = abs(grocer_fp-grocer_fret);
       grocer_ibig = grocer_i;
    end
    grocer_i = grocer_i+1;
  end
  // end while
 
  if 2*abs(grocer_fp-grocer_fret)<=grocer_tol*(abs(grocer_fp)+...
     abs(grocer_fret)) then
    grocer_hessf = hessian(grocer_func,grocer_xit,varargin(:))
    grocer_niter = grocer_iter;
    result = tlist(['resulmin';'meth';'b';'f';'hess';'iter';'flag'],...
            'pow',grocer_pin,grocer_fret,grocer_hessf,grocer_niter,0)
    return
 
  end
 
  grocer_ptt = 2*grocer_pin-grocer_pt;
  grocer_xit = grocer_pin-grocer_pt;
  grocer_pt = grocer_pin;
  grocer_pin = grocer_ptt;
  grocer_fptt = grocer_func(grocer_pin,varargin(:))
 
  if grocer_fptt<grocer_fp then
     grocer_t = 2*(grocer_fp-2*grocer_fret+grocer_fptt)*...
     ((grocer_fp-grocer_fret-grocer_del)^2)-grocer_del*((grocer_fp-grocer_fptt)^2);
  end
 
  if grocer_t<0 then
     [grocer_pin grocer_fret] = linmin(grocer_pin,grocer_xit,grocer_tol,...
        grocer_func,varargin(:))
    grocer_xi(:,grocer_ibig) = grocer_xit;
  end
end
// end of while
 
grocer_hessf = evstr(grocer_chg)
 
result = tlist(['results';'meth';'b';'f';'hess';'iter';'flag'],...
            'pow',grocer_pin,grocer_fret,grocer_hessf,grocer_niter,0)
endfunction
