function res=ppooled_hac1(y,index,x,typvcv)
 
// PURPOSE: performs Pooled HAC Least Squares for Panel Data
// (for balanced data)
// ------------------------------------------------------------
// INPUT:
// * y = a (nobs*nindiv x 1) matrix of all of the individual's
//   observations vertically concatenated. This matrix must
//   include in the first column the dependent variable, the
//   independent variables must follow accordingly.
// * index = index vector that identifies each observation with
//   an individual
//   e.g. 1  (first 2 observations  for individual # 1)
//        1
//        2  (next  1 observation   for individual # 2)
//        3  (next  3 observations  for individual # 3)
//        3
//        3
// * x = a (nobs*nindiv x k) matrix of exogenous variables
// * typvcv = 1 or 2 with
//    - 1 "clustered" covariance matrix of Arellano (1987)
//      recommended when T is fixed and N large
//      but also "works" when T is large and N fixed
//      see Hansen C. B. (2007) (reference below)
//    - 2 a Newey-west type estimator 
//      recommended when T is large and N fixed
// ------------------------------------------------------------
// OUTPUT:
// res = a tlist with
//   . res('meth')  = 'panel pooled'
//   . res('y')     = y data vector
//   . res('x')     = x data matrix
//   . res('nobs')  = nobs
//   . res('nvar')  = nvars
//   . res('beta')  = bhat
//   . res('yhat')  = yhat
//   . res('resid') = residuals
//   . res('vcovar') = estimated variance-covariance matrix of
//     beta
//   . res('sige')  = estimated variance of the residuals
//   . res('sige')  = estimated variance of the residuals
//   . res('ser')  = standard error of the regression
//   . res('tstat') = t-stats
//   . res('pvalue') = pvalue of the betas
//   . res('condindex') = multicolinearity cond index
//   . res('prescte') = boolean indicating the presence or
//     absence of a constant in the regression
//   . res('rsqr')  = rsquared
//   . res('rbar')  = rbar-squared
//   . res('f')    = F-stat for the nullity of coefficients
//     other than the constant
//   . res('pvaluef') = its significance level
// ------------------------------------------------------------
// REFERENCE:
// * Arellano M. (2003), "Panel Data Econometrics", ed. OUP
// * Hansen C. B. (2007), "Asymptotic Properties of a Robust Variance
//    Matrix for Panel Data when T is Large", Journal of Econometrics,
//    141, 597-620.   
// * Green W. H. (2000), "Econometric Analysis", 4th ed, Prentice Hall
// ------------------------------------------------------------
// Copyright Emmanuel Michaux 2010
// http://dubois.ensae.net/grocer.html 
 
[nargout,nargin]=argn(0)
if nargin < 4 then
  typvcv=1;
end
[nobs,nx]=size(x);
 
uniq=-gsort(-unique(index));
nindiv=size(uniq,1);

bhat=ols0(y,x);
yhat=x*bhat;
resid=y-yhat;
sigu=resid'*resid;
sige=sigu/(nobs-nx);
ser=sqrt(sige);
condindex=bkwols(x);
llike=-0.5*nobs*(log(2*%pi)+log(sigu/nobs)+1);

Ti=[];
for i=1:nindiv
  id_i=find(index==uniq(i));
  Ti=[Ti;size(id_i,'*')];
end

//"clustered" covariance matrix of Arellano (1987)
if typvcv==1 then 
  xx=[];
  xuux=[]; 
  for i=1:nindiv
    id_i=find(index==uniq(i));
    eps_i=resid(id_i);
    x_i=x(id_i,:);
    xx=xx+x_i'*x_i;
    xuux=xuux+x_i'*eps_i*eps_i'*x_i;
  end 
  vcovar=inv(xx)*xuux*inv(xx);
else 
// Newey-West type variance matrix
// see Arellano M. (2003) p. 19
  if length(unique(Ti))==1;
    win=floor(5*Ti(1)^0.25);
    idx=1:Ti(1):nobs;   
    omeg=0
    for t=1:Ti(1)
      omeg=omeg+x(idx+t-1,:)'*resid(idx+t-1)*resid(idx+t-1)'*x(idx+t-1,:);
    end 

    for l=1:win
      omegl=[]
      for t=(l+1):Ti(1)
        omegl=omegl+x(idx+t-1,:)'*resid(idx+t-1)*resid(idx+t-l-1)'*x(idx+t-1-l,:);
      end 
      omeg=omeg+(1-l/(win+1))*(omegl+omegl');
    end 
  else
    error('Newey-West type variance matrix does''nt work with unbalanced panel');
  end 
  vcovar=inv(x'*x)*omeg*inv(x'*x);
end 

df=nobs-nx;
tmp=diag(vcovar);
tstat=bhat./sqrt(tmp);
pvalue=ones(nx,1);
for i=1:nx
  pvalue(i)=(1-cdft("PQ",abs(tstat(i)),df))*2;
end

if typvcv==1 then 
  typvcv='clustered'
elseif typvcv==2 then 
  typvcv='Newey-West'
end 

res=tlist(['results';'meth';'y';'x';'nobs';'nvar';'beta';'yhat';...
'resid';'vcovar';'sige';'sigu';'ser';'tstat';'pvalue';'condindex';...
'prescte';'llike';'hac']...
,'panel pooled',y,x,nobs,nx,bhat,yhat,resid,vcovar,sige,sigu,ser,tstat,...
pvalue,condindex,%t,llike,typvcv);

indcte=search_cte(x);
prescte=~isempty(indcte);
if prescte & nx ~=1 then
// there is a constant and at least another exogenous variable:
// R² and Rbar² make sense
  res=add_r2(res,sigu,y-mean(y),nobs,nx)
end 

endfunction
