function [rprobit]=probit(grocer_namey0,varargin)
 
// PURPOSE: computes Probit Regression
// ------------------------------------------------------------
// References: Arturo Estrella (1998) 'A new measure of fit
// for equations with dichotmous dependent variable', JBES,
// Vol. 16, #2, April, 1998.
// ------------------------------------------------------------
// INPUT:
// * grocer_namey0 = a time series, a real (nx1) vector or a
// string equal to the name of a time series or a (nx1) real
//  vector between quotes; all values should be 0 or 1
// * varargin = arguments which can be:
//   . a time series
//   . a real (nxp) vector
//   . a string equal to the name of a time series or a (nx1)
//     real vector between quotes
//   . the string 'noprint' if the user doesn't want the
//     to print the results of the regression
//   . the string 'maxit=xx' if the user wants to set the
//     maximum # of iterations to xx (default=100)
//   . the string 'tol=xx' if the user wants to set the
//     convergence criterion to xx (default=1e-6)
//   . the string 'dropna' if the user wants to use in the
//     regression all dates with no NA value in any variable
//     (the main use of this option should be when dealing with
//     daily ts)
// ------------------------------------------------------------
// OUTPUT:
// rprobit = a results tlist with
//   . rprobit('meth')  = 'probit'
//   . rprobit('y')     = y data vector
//   . rprobit('x')     = x data matrix
//   . rprobit('nobs')  = # observations
//   . rprobit('nvar')  = # variables
//   . rprobit('beta')  = bhat
//   . rprobit('yhat')  = yhat
//   . rprobit('resid') = residuals
//   . rprobit('vcovar') = estimated variance-covariance matrix of
//     beta
//   . rprobit('tstat') = t-stats
//   . rprobit('pvalue') = pvalue of the betas
//   . rprobit('r2mf')  =  = McFadden pseudo-R
//   . rprobit('rsqr')  =  = Estrella R
//   . rprobit('lratio') = LR-ratio test against intercept model
//   . rprobit('lik')    = unrestricted Likelihood
//   . rprobit('zip')    = # of 0's
//   . rprobit('one)    = # of 1's
//   . rprobit('iter')   = # of iterations
//   . rprobit('crit')   = convergence criterion
//   . rprobit('namey') = name of the y variable
//   . rprobit('namex') = name of the x variables
//   . rprobit('prests') = boolean indicating the presence or
//     absence of a time series in the regression
//   . rprobit('prescte') = %f (for printings)
//   . rprobit('bounds') = if there is a timeseries in the
//     regression, the bounds of the regression
//   . rprobit('dropna') = boolean indicating if NAs have
//		   been dropped
//   . rprobit('nonna') = vector indicating position of non-NAs
// ------------------------------------------------------------
// PRINTS: If the user has not given the argument 'noprint',
// the results of the regression and various diagnostics
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002-2007
// http://dubois.ensae.net/grocer.html
 
// adapted from:
// James P. LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jpl@jpl.econ.utoledo.edu
 
 
grocer_tol = 1e-8;
grocer_maxit = 500;
grocer_prt=%t
grocer_dropna=%f
 
// separate from the list of variable arguments the list of
// exogenous variables and options if any
grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   if typeof(varargin(grocer_i)) == 'string' then
      if varargin(grocer_i) == 'noprint' then
         grocer_prt=%f
         varargin(grocer_i)=null()
      elseif varargin(grocer_i) == 'dropna' then
         grocer_dropna=%t
         varargin(grocer_i)=null()
      elseif part(varargin(grocer_i),1:3) == 'tol' |...
         part(varargin(grocer_i),1:5) == 'maxit' then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null()
      end
   elseif typeof(varargin(grocer_i)) ~= 'constant' then
      error('invalid type: '+typeof(varargin(grocer_i)))
   end
end
 
grocer_lx=varargin
[grocer_y,grocer_namey,grocer_x,grocer_namexos,grocer_prests,grocer_boundsvarb,nonna]=...
  explouniv(grocer_namey0,grocer_lx,[],['endogenous';'exogenous'],%t,grocer_dropna)
 
// check for all 1's or all 0's
tmp = matrix(find(grocer_y==1),1,-1);
chk = max(size(tmp));
nobs = size(grocer_y,1);
if chk==nobs then
  error('probit: y-vector contains all ones');
elseif chk==0 then
  error('probit: y-vector contains no ones');
end
 
[nobs,nvar] = size(grocer_x);
[nobs2] = size(grocer_y,1);
 
if nobs~=nobs2 then
  error('x and y must have same # obs');
end
if nvar >= nobs then
   error('too many exogenous variables')
end
 
// maximum likelihood probit estimation
 
 
b = ols0(grocer_y,grocer_x);
// use ols values as start
 
crit = 1;
i = ones(nobs,1);
tmp1 = zeros(nobs,nvar);
tmp2 = zeros(nobs,nvar);
 
iter = 1;
 
while (iter<=grocer_maxit)&(crit>grocer_tol) then
  pdf = norm_pdf(grocer_x*b);
  cdf = cdfnor("PQ",grocer_x*b,zeros(nobs,1),ones(nobs,1));
 
  tmp = matrix(find(cdf<=0),1,-1);
  [n1,n2] = size(tmp);
  if n1~=0 then
    cdf(tmp,1) = .00001*ones(max(size(tmp)),1);
  end
 
  tmp = matrix(find(cdf>=1),1,-1);
  [n1,n2] = size(tmp);
  if n1~=0 then
    cdf(tmp,1) = .99999*ones(max(size(tmp)),1);
  end
 
 
  // gradient vector for probit (and probit)
  term1 = grocer_y .* (pdf ./ cdf);
  term2 = (i-grocer_y) .* (pdf ./ (i-cdf));
 
// Bug fix: Mauro Barrera (may 2005)
 
  for kk = 1:nvar
    tmp1(:,kk) = term1 .* grocer_x(:,kk);
    tmp2(:,kk) = term2 .* grocer_x(:,kk);
  end
 
  g = tmp1-tmp2;
 
  gs = sum(g,1)';
 
  // compute see page 883 Green, 1997
  q = 2*grocer_y-i;
  xxb = grocer_x*b;
  pdf = norm_pdf(q .* xxb);
  cdf = cdfnor("PQ",q .* xxb,zeros(nobs,1),ones(nobs,1));
  lambda = q .* pdf ./ cdf;
  H = zeros(nvar,nvar);
  for ii = 1:nobs
    xb = grocer_x(ii,:)*b;
    xp = grocer_x(ii,:)';
    H = H-lambda(ii,1)*(lambda(ii,1)+xb)*(xp*grocer_x(ii,:));
  end
 
  db = -inv(H)*gs;
  // stepsize determination
  s = 2;
  term1 = 0;
  term2 = 1;
  while term2>term1 then
    s = s/2;
    term1 = pr_like(b+s*db,grocer_y,grocer_x);
    term2 = pr_like(b+s*db/2,grocer_y,grocer_x);
  end
 
  bn = b+s*db;
  crit = abs(max(max(db,'r')));
  b = bn;
  iter = iter+1;
 
end
// end of while
 
if iter>=grocer_maxit then
  write(%io(2),'probit: no convergence in '+string(iter)+...
  ' iterations')
end
 
// compute Hessian for inferences
q = 2*grocer_y-i;
// see page 883 Green, 1997
xxb = grocer_x*b;
pdf = norm_pdf(q .* xxb);
cdf = cdfnor("PQ",q .* xxb,zeros(nobs,1),ones(nobs,1));
lambda = q .* pdf ./ cdf;
H = zeros(nvar,nvar);
for i = 1:nobs
  xb = grocer_x(i,:)*b;
  xp = grocer_x(i,:)';
  H = H-lambda(i,1)*(lambda(i,1)+xb)*(xp*grocer_x(i,:));
end
 
// now compute regression results
covb = -inv(H);
stdb = sqrt(diag(covb));
tstat = b ./ stdb
 
df=nobs-nvar
pvalue=ones(nvar,1)
for i=1:nvar
   pvalue(i)=(1-cdft("PQ",abs(tstat(i)),df))*2
end
 
// fitted probabilities
yhat = cdfnor("PQ",grocer_x*b,zeros(nobs,1),ones(nobs,1))
resid = grocer_y-yhat
sige = resid'*resid/nobs
 
// find ones
tmp = matrix(find(grocer_y==1),1,-1);
P = max(size(tmp));
cnt0 = nobs-P;
cnt1 = P;
// proportion of 1's
P = P/nobs
// restricted likelihood
like0 = nobs*(P*log(P)+(1-P)*log(1-P));
// unrestricted Likelihood
like1 = pr_like(b,grocer_y,grocer_x);
 
// McFadden pseudo-R2
r2mf = 1-abs(like1)/abs(like0)
// Estrella R2
term0 = 2/nobs*like0;
term1 = 1/(abs(like1)/abs(like0)^term0);
rsqr = 1-term1
 
// LR-ratio test against intercept model
lratio = 2*(like1-like0)
 
rprobit = tlist(['results';'meth';'y';'x';'nobs';'nvar';...
'beta';'yhat';'resid';'vcovar';'tstat';'pvalue';'r2mf'...
;'rsqr';'lratio';'llike';'zip';'one';'iter';'crit';'namey'...
;'namex';'prests';'prescte';'dropna';'condindex'],'probit',grocer_y,grocer_x,nobs,nvar,b,yhat...
,resid,covb,tstat,pvalue,r2mf,rsqr,lratio,like1,cnt0,cnt1,...
iter,crit,grocer_namey,grocer_namexos,grocer_prests,%f,grocer_dropna,bkwols(grocer_x))
 
if grocer_prests then
   rprobit(1)($+1) = 'bounds'
   rprobit('bounds')=grocer_boundsvarb
end
 
if grocer_dropna then
   rprobit(1)($+1)='nonna'
   rprobit('nonna')=nonna
end
 
if grocer_prt then
   prtuniv(rprobit,%io(2))
end
 
endfunction
