function [rtsls]=iv1(y,y1,x1,xall)
 
// PURPOSE: compute instrumental variables estimation when all
// variables are in vector or matrix form
// ------------------------------------------------------------
// INPUT:
// * y = dependent variable vector (nobs x 1)
// * y1 = endogenous variables matrix (nobs x g) for this
//   equation
// * xexog  = exogenous variables matrix for this equation
// * xall   = instruments for variables y1
// ------------------------------------------------------------
// OUTPUT:
// results = a structure tlist with
// - rtsls('meth')  = 'tsls'
// - rtsls('nobs')  = nobs
// - rtsls('nendog') = # of endogenous
// - rtsls('nexog')  = # of exogenous
// - rtsls('nvar')   = # of endogenous + # of exogenous
// - rtsls('y')      = y data vector
// - rtsls('beta')  = bhat estimates
// - rtsls('tstat') = t-statistics
// - rtsls('yhat')  = yhat predicted values
// - rtsls('resid') = residuals
// - rtsls('residtsls') = residuals calculated with the
//   endogenous variables replaced by their regression from
//   first stage estimation
// - rtsls('sigu')  = e'*e
// - rtsls('sige')  = e'*e/(n-k)
// - rtsls('dw')    = Durbin-Watson Statistic
// - rtsls('prescte') = boolean indicating the presence or
//     absence of a constant in the regression
// - rtsls('rsqr')  = rsquared
// - rtsls('rbar')  = rbar-squared
// - rtsls('f')    = F-stat for the nullity of coefficients
//    other than the constant
// - rtsls('pvaluef') = its significance level
// - rtsls('grsqr')  = generalized rsquared (that is which takes
//    into account the endogeneity of some explicative
//    variables)
// ------------------------------------------------------------
// Copyright Eric Dubois 2002
// adapted from:
// James P. LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jlesage@spatial-econometrics.com
 
[nargout,nargin] = argn(0)
if nargin~=4 then
  error('Wrong # of arguments to iv1');
end
 
[nobs1,g] = size(y1);
[nobs2,k] = size(x1);
[nobs3,l] = size(xall);
nvar=g+k
 
if nobs2==nobs3 & (nobs1==nobs2 | nobs1==0 ) then
    if nobs2==nobs3 then
      nobs = nobs2;
   end
else
   error('iv1: # of observations in yendog, xexog, xall not the same');
end
 
if nvar >= nobs then
   error('too many exogenous variables')
end
 
// xall contains all explanatory variables
// x1 contains exogenous
// y1 contains endogenous
 
if nobs1 ~= 0 then
   y1hat=xall*ols0(y1,xall)
else
   y1hat=[]
end
xhat=[y1hat x1]
[bhat,xpxi]=ols0(y,xhat)
 
// yhat
yhat = [y1,x1]*bhat
// residuals
resid = y-yhat
residiv = y - xhat*bhat
sigu = resid'*resid;
sige = sigu/(nobs-k-g)
ser=sqrt(sige)
// sige
tmp = sige*diag(xpxi);
tstat = bhat ./ sqrt(tmp)
 
df=nobs-nvar
pvalue=ones(nvar,1)
for i=1:nvar
   pvalue(i)=(1-cdft("PQ",abs(tstat(i)),df))*2
end
 
condindex=bkwols([y1,x1])
 
prescte=%f
if nvar ~= 1 then
   i=1
   while (i <= nvar) & ~prescte then
      // if all values are equal to the first one then,
      // the variable is constant
      prescte=and(xhat(:,i) == xhat(1,i))
      i=i+1
   end
end
 
ediff = resid(2:nobs)-resid(1:nobs-1);
dw = ediff'*ediff/sigu
// durbin-watson
 
rtsls = tlist(['results';'meth';'nobs';'nendog';'nexog';'nvar';...
'y';'x';'beta';'tstat';'pvalue';'yhat';'resid';'residiv';'sigu';...
'sige';'ser';'dw';'prescte';'condindex'],...
'iv',nobs,g,k,nvar,y,xhat,bhat,tstat,pvalue,yhat,resid,residiv,...
sigu,sige,ser,dw,prescte,condindex)
 
ym = y-mean0(y);
if prescte then
// there is a constant and at least another exogenous variable:
// R and Rbar make sense
   df=nobs-nvar
   rsqr2 = ym'*ym;
// r-squared
 
   rtsls(1)($+1)='grsqr'
   rtsls(1)($+1)='grbar'
   grsqr=1-residiv'*residiv/(ym'*ym)
   rtsls('grsqr')=grsqr
   rtsls('grbar')=1-residiv'*residiv/(ym'*ym)/(nobs-nvar)*(nobs-1)
   if grsqr <0 then
      f=%nan
   elseif grsqr ~= 1 then
      f=grsqr/(1-grsqr)*df/(nvar-1)
   else
      warning('generalized rsqr = 1: your exogenous variables are exactly colinear')
      f=%inf
   end
   pvaluef=1-cdff("PQ",f,nvar-1,nobs-nvar)
   rtsls(1)($+1)='f'
   rtsls('f') = f
   rtsls(1)($+1)='pvaluef'
   rtsls('pvaluef') = pvaluef
 
end
 
endfunction
 
