  function [rols]=ols(grocer_namey0,varargin)
 
// PURPOSE: the most general function performing least-squares
// regression
// ------------------------------------------------------------
// INPUT:
// * grocer_namey0 = a time series, a real (nx1) vector or a
// string equal to the name of a time series or a (nx1) real
// vector between quotes
// * varargin = arguments which can be:
//   . a time series
//   . a real (nxp) vector
//   . a string equal to the name of a time series or a (nxp)
//     real vector between quotes
//   . the string 'noprint' if the user doesn't want to print
//     the results of the regression
//   . 'dropna' if the user wants to remove the NA values
//     from the data
// ------------------------------------------------------------
// OUTPUT:
// rols = a results tlist with
//   . rols('meth')  = 'ols'
//   . rols('y')     = y data vector
//   . rols('x')     = x data matrix
//   . rols('nobs')  = # observations
//   . rols('nvar')  = # variables
//   . rols('beta')  = bhat
//   . rols('yhat')  = yhat
//   . rols('resid') = residuals
//   . rols('vcovar') = estimated variance-covariance matrix of
//     beta
//   . rols('sige')  = estimated variance of the residuals
//   . rols('sigu')  = sum of squared residuals
//   . rols('ser')  = standard error of the regression
//   . rols('tstat') = t-stats
//   . rols('pvalue') = pvalue of the betas
//   . rols('dw')    = Durbin-Watson Statistic
//   . rols('condindex') = multicolinearity cond index
//   . rols('prescte') = boolean indicating the presence or
//     absence of a constant in the regression
//   . rols('llike') = the log-likelihood
//   . rols('rsqr')  = rsquared
//   . rols('rbar')  = rbar-squared
//   . rols('f')    = F-stat for the nullity of coefficients
//     other than the constant
//   . rols('pvaluef') = its significance level
//   . rols('like') = log-likelihood of the regression
//   . rols('prests') = boolean indicating the presence or
//     absence of a time series in the regression
//   . rols('namey') = name of the y variable
//   . rols('namex') = name of the x variables
//   . rols('dropna') = boolean indicating if NAs have
//		   been dropped
//   . rols('bounds') = if there is a timeseries in the
//     regression, the bounds of the regression
//   . rols('nonna') = vector indicating position of non-NAs
// ------------------------------------------------------------
// PRINTS: If the user has not given the argument 'noprint',
// the results of the regression and various diagnostics
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002-2007
// http://dubois.ensae.net/grocer.html
 
// set defaults
[grocer_dropna,grocer_prt,varargin]=vararg2dropnaprt(varargin(:))
 
[y,namey,x,namexos,prests,boundsvarb,nonna]=...
explouniv(grocer_namey0,varargin,[],['endogenous';'exogenous'],%t,grocer_dropna)
 
// provides the results from the regression of the vector y
// on the vector x
rols=ols2(y,x)
 
// saves the names, the bounds if the regression involves ts
rols(1)($+1) = 'prests'
rols(1)($+1) = 'namex'
rols(1)($+1) = 'namey'
rols(1)($+1) = 'dropna'
rols('prests')=prests
rols('namex')=namexos
rols('namey')=namey
rols('dropna')=grocer_dropna
 
if prests then
   rols(1)($+1) = 'bounds'
   rols('bounds')=boundsvarb
end
 
if grocer_dropna then
   rols(1)($+1)='nonna'
   rols('nonna')=nonna
end
 
if grocer_prt then
// the user has not entered 'noprint' as an argument
// a command to activate if foutput has been previously definded:
// prtuniv(rols,foutput)
   prtuniv(rols,%io(2))
   pltuniv(rols,'all')
end
 
endfunction
