function res=olsarma(grocer_AR,grocer_MA,grocer_namey,varargin)
 
// PURPOSE: perform the estimation of an ols model with arma
// errors
// ------------------------------------------------------------
// INPUT:
// * grocer_AR = a (nar x 1) or (1 x nar) string or real vector
//   of parameters corresponding to the AR part of the error
//   process
//     . if AR is a real then all parameters are estimated
//     . if AR is a string then all parameters with in AR
//     with an equality (such as '=0.5') are constrained to 
//     the given value (0.5 in the example)
//     . if AR is a string then it can contain inequality
//    constraints; for instance '<0.5' indicates that coeff
//    must be lower than 0.5
//     . if initown is set to %F, then the user can give any
//     value to AR; only it size matters for the estimation
//     process
//    . if initown is set to %F,
// * grocer_MA = a (nmaf x 1) or (1 x nmaf) string or real
//   vector of corresponding to the AR part of the error, with
//   the same working as for AR
// * grocer_namey = a time series, a real (nx1) vector or a
// string equal to the name of a time series or a (nx1) real
// vector between quotes
// * varargin = arguments which can be:
//   . a time series
//   . a real (nxp) vector
//   . a string equal to the name of a time series or a (nxp)
//     real vector between quotes
//   . the string 'noprint' if the user doesn't want to print
//     the results of the regression
//   . 'dropna' if the user wants to remove the NA values
//     from the data
//   . 'init=own' if the user wants the function to impose
//     starting values for the parameters
//   . 'beta=xxx' to fix the starting values of the
//     coefficients of the regression if the user has given the
//     option 'init=own'
// ------------------------------------------------------------
// OUTPUT:
// res = a results tlist with
//   . res('meth')  = 'ols with arma errors'
//   . res('y')     = y data vector
//   . res('x')     = x data matrix
//   . res('nobs')  = # observations
//   . res('nvar')  = # variables
//   . res('beta')  = bhat
//   . res('yhat')  = yhat
//   . res('resid') = residuals
//   . res('vcovar') = estimated variance-covariance matrix of
//     beta
//   . res('sige')  = estimated variance of the residuals
//   . res('sigu')  = sum of squared residuals
//   . res('ser')  = standard error of the regression
//   . res('tstat') = t-stats
//   . res('pvalue') = pvalue of the betas
//   . res('dw')    = Durbin-Watson Statistic
//   . res('condindex') = multicolinearity cond index
//   . res('prescte') = boolean indicating the presence or
//     absence of a constant in the regression
//   . res('llike') = the log-likelihood
//   . res('AR') = the estimated AR part of the residuals
//   . res('MA') = the estimated MA part of the residuals
//   . res('tAR') = the t-statistics of the AR part of the
//     residuals
//   . res('tMA') = the t-statistics of the MA part of the
//     residuals
//   . res('pvalues AR') = the p-values of the AR part of the
//     residuals
//   . res('pvalues MA') = the p-values of the MA part of the
//     residuals
//   . res('V') = the estimated variance of the innovations of
//     the residuals
//   . res('AIC') = the value of the Akaïke Critrium
//   . res('BIC') = the value of the Schwarz Critrium
//   . res('grad') = the gradient at solution
//   . res('type') = the e4 type of the model
//   . res('prests') = boolean indicating the presence or
//     absence of a time series in the regression
//   . res('namey') = name of the y variable
//   . res('namex') = name of the x variables
//   . res('dropna') = boolean indicating if NAs have
//		   been dropped
//   . res('bounds') = if there is a timeseries in the
//     regression, the bounds of the regression
//   . res('nonna') = vector indicating position of non-NAs
// ------------------------------------------------------------
// Copyright: Eric Dubois 2009-2010
// http://dubois.ensae.net/grocer.html
 
grocer_dropna=%f
grocer_prt=%t
grocer_initown=%f
grocer_beta=[]

grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   grocer_argi=varargin(grocer_i)
   if typeof(grocer_argi) == 'string' then
      grocer_argin=strsubst(grocer_argi,' ','')
 
      if grocer_argin == 'noprint' then
         grocer_prt=%f
         varargin(grocer_i) =null()
      elseif grocer_argin == 'dropna' then
         grocer_dropna=%t
         varargin(grocer_i) =null()
      elseif part(grocer_argin,1:5) == 'beta=' then
         execstr('grocer_'+grocer_argi)
         varargin(grocer_i) =null()
      elseif grocer_argin == 'init=own' then
         grocer_initown=%t
         varargin(grocer_i) =null()
      end
   end
end

if grocer_initown & isempty(grocer_beta) then
   error('with the option ''init=own'' you must enter starting values for beta (option ''beta=...'')')
end

[y,namey,x,namexos,prests,boundsvarb,nonna]=...
explouniv(grocer_namey,varargin,[],['endogenous';'exogenous'],%t,grocer_dropna)

res=olsarma1(grocer_AR,grocer_MA,y,x,grocer_initown,namexos,grocer_beta)

// saves the names, the bounds if the regression involves ts
res(1)($+1) = 'prests'
res(1)($+1) = 'namex'
res(1)($+1) = 'namey'
res(1)($+1) = 'dropna'
res('prests')=prests
res('namex')=namexos
res('namey')=namey
res('dropna')=grocer_dropna
 
if prests then
   res(1)($+1) = 'bounds'
   res('bounds')=boundsvarb
end
 
if grocer_dropna then
   res(1)($+1)='nonna'
   res('nonna')=nonna
end
 
if grocer_prt then
// the user has not entered 'noprint' as an argument
// a command to activate if foutput has been previously definded:
// prtuniv(res,foutput)
   prtuniv(res,%io(2))
   pltuniv(res,'all')
end

 
endfunction
 
