function [rrobust]=robust(wfunc,wparm,grocer_namey0,varargin)
 
// PURPOSE: robust regression using iteratively reweighted
//          least-squares
// ------------------------------------------------------------
// INPUT:
// * wfunc = 'huber' for Huber's t function
//           'ramsay' for Ramsay's E function
//           'andrew' for Andrew's wave function
//           'tukey' for Tukey's biweight
// * wparm = weighting function parameter
// * grocer_namey0 = a time series, a real (nx1) vector or a
// string equal to the name of a time series or a (nx1) real
// vector between quotes
// * varargin = arguments which can be:
//   . a time series
//   . a real (nxp) vector
//   . a string equal to the name of a time series or a (nxp)
//     real vector between quotes
//   . the string 'noprint' if the user doesn't want to print
//     the results of the regression
//   . the string 'dropna' if the user wants to use in the
//     regression all dates with no NA value in any variable
//     (the main use of this option should be when dealing with
//     daily ts)
// ------------------------------------------------------------
// OUTPUT:
// rrobust = a tlist with
//   . rrobust('meth')  = 'robust'+ 'huber', 'ramsay', 'andrew'
//                         or 'tukey'
//   . rrobust('y')     = y data vector
//   . rrobust('x')     = x data matrix
//   . rrobust('nobs')  = nobs
//   . rrobust('nvar')  = nvars
//   . rrobust('beta')  = bhat
//   . rrobust('yhat')  = yhat
//   . rrobust('resid') = residuals
//   . rrobust('vcovar') = estimated variance-covariance matrix
//                         of beta
//   . rrobust('sige')  = estimated variance of the residuals
//   . rrobust('sige')  = estimated variance of the residuals
//   . rrobust('ser')  = standard error of the regression
//   . rrobust('tstat') = t-stats
//   . rrobust('pvalue') = pvalue of the betas
//   . rrobust('dw')    = Durbin-Watson Statistic
//   . rrobust('prescte') = boolean indicating the presence or
//     absence of a constant in the regression
//   . rrobust('rsqr')  = rsquared
//   . rrobust('rbar')  = rbar-squared
//   . rrobust('f')    = F-stat for the nullity of coefficients
//     other than the constant
//   . rrobust('pvaluef') = its significance level
//   . rrobust('wparm') = wparm
//   . rrobust('iter')  = # of iterations
//   . rrobust('weight') = nobs - vector of weights
//   . rrobust('convg') = convg criterion
//   . rrobust('prests') = boolean indicating the presence or
//     absence of a time series in the regression
//   . rrobust('namey') = name of the y variable
//   . rrobust('namex') = name of the x variables
//   . rrobust('dropna') = boolean indicating if NAs have
//		   been dropped
//   . rrobust('bounds') = if there is a timeseries in the
//     regression, the bounds of the regression
//   . rrobust('nonna') = vector indicating position of non-NAs
// ------------------------------------------------------------
// PRINTS: If the user has not given the argument 'noprint',
// the results of the regression and various diagnostics
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002
// http://dubois.ensae.net/grocer.html
 
// adapted from:
// James P. LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jlesage@spatial-econometrics.com
 
// separate from the list of variable arguments the list of
// exogenous variables and 'noprint' if the user has given this
// argument
grocer_dropna=%f
grocer_prt=%t
grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   grocer_argi=varargin(grocer_i)
   if typeof(grocer_argi) == 'string' then
      grocer_argi=strsubst(grocer_argi,' ','')
      if grocer_argi == 'noprint' then
         grocer_prt=%f
         varargin(grocer_i)=null()
      elseif grocer_argi == 'dropna' then
         grocer_dropna=%t
         varargin(grocer_i)=null()
      end
   end
end
 
grocer_lx=varargin
[grocer_y,grocer_namey,grocer_x,grocer_namexos,grocer_prests,grocer_boundsvarb,nonna]=...
    explouniv(grocer_namey0,grocer_lx,[],['endogenous';'exogenous'],%t,grocer_dropna)
 
nobs2=size(grocer_y,1)
[nobs,nvar] = size(grocer_x);
if nvar >= nobs then
   error('too many exogenous variables')
end
if nobs~=nobs2 then
  error('x and y must have same # obs in ols2');
end
 
 
// find starting values
w = ones(nobs,1);
bhat = ols0(grocer_y,grocer_x)
yhat = grocer_x*bhat;
resid = grocer_y-yhat;
 
scale = median(abs(resid-median(resid)))/.6745;
 
convg = 1;
cnt = 0;
 
while convg>.00001 then
 
  bhato = bhat;
  resid = resid/scale;
  cnt = cnt+1;
 
  select wfunc
  case 'huber' then
    // Huber's t function
    w = wparm*ones(nobs,1) ./ abs(resid);
    wi = matrix(find(abs(resid)<=wparm),1,-1);
    w(wi) = ones(max(size(wi)),1)
 
  case 'ramsay' then
    // Ramsay's E function
    w = exp(-wparm*abs(resid));
    wi = matrix(find(resid==0),1,-1);
    w(wi) = ones(max(size(wi)),1)
 
  case 'andrew' then
    // Andrew's wave function
    w = sin(resid/wparm) ./ (resid/wparm);
    wi = matrix(find(resid==0),1,-1);
    w(wi) = ones(max(size(wi)),1)
    wi = matrix(find(resid>%pi*wparm),1,-1);
    w(wi) = zeros(max(size(wi)),1)
 
  case 'tukey' then
    // Tukey's biweight
    w = (1-(resid/wparm).^2).^2;
    wi = matrix(find(resid==0),1,-1);
    w(wi) = ones(max(size(wi)),1)
    wi = matrix(find(abs(resid)>wparm),1,-1);
    w(wi) = zeros(max(size(wi)),1)
 
  else
    error('incorrect weight function option');
  end
 
  // do weighted least-squares
  ystar = grocer_y .* sqrt(w);
  xstar = grocer_x .* (ones(1,nvar) .*. sqrt(w));
  [bet,xpxi]=ols0(ystar,xstar)
  resid = grocer_y-grocer_x*bet;
 
  // check for convergence
  convg = max(abs(bhat-bhato) ./ abs(bhato));
 
end
// end of while loop
 
yhat=grocer_x*bet
resid=grocer_y-yhat
sigu =resid'*resid
sige= sigu/(nobs-nvar)
ser=sqrt(sige)
vcovar = sige*inv(xstar'*xstar)
tstat=bet ./ sqrt(diag(vcovar))
 
ym = grocer_y-mean0(grocer_y);
ediff = resid(2:nobs)-resid(1:nobs-1)
dw = ediff'*ediff/sigu
// durbin-watson
 
df=nobs-nvar
pvalue=ones(nvar,1)
for i=1:nvar
   pvalue(i)=(1-cdft("PQ",abs(tstat(i)),df))*2
end
 
condindex=bkwols(grocer_x)
 
 
rrobust = tlist(['results';'meth';'y';'x';'nobs';'nvar'...
;'beta';'yhat';'resid';'vcovar';'sige';'sigu';'ser'...
;'tstat';'pvalue';'dw';'condindex';'prescte';'wparm'...
;'iter';'weight';'convg'],'robust '+wfunc,grocer_y,grocer_x,nobs,nvar...
,bet,yhat,resid,vcovar,sige,sigu,ser,tstat,pvalue,dw,...
condindex,%f,wparm,cnt,w,convg)
 
// Note: R� has no sense here since the regression is not OLS
// (it can be negative for instance !)
 
// saves the names, the bounds if the regression involves ts
rrobust(1)($+1) = 'prests'
rrobust(1)($+1) = 'namex'
rrobust(1)($+1) = 'namey'
rrobust(1)($+1) = 'dropna'
rrobust('prests')=grocer_prests
rrobust('namex')=grocer_namexos
rrobust('namey')=grocer_namey
rrobust('dropna')=grocer_dropna
if grocer_prests then
   rrobust(1)($+1) = 'bounds'
   rrobust('bounds')=grocer_boundsvarb
end
if grocer_dropna then
   rrobust(1)($+1)='nonna'
   rrobust('nonna')=nonna
end
 
if grocer_prt then
// the user has not entered 'noprint' as an argument
// a command to activate if foutput has been previously defined:
// prtuniv(rrobust,foutput)
   prtuniv(rrobust,%io(2))
end
endfunction
