function [grocer_mats,grocer_names,grocer_prests,grocer_b,grocer_nonna]=explon(grocer_vars,grocer_named,grocer_b,grocer_testna,grocer_dropna,grocer_mindat)
 
// PURPOSE: explode n lists of variables into matrices, names
// and define corresponding bounds
// ------------------------------------------------------------
// INPUT:
// * grocer_vars = list of n:
//   - lists of variables
//   - matrix of names of variables
//   the variables being in the form of
//   * a time series
//   * a real vector,
//   * a real matrix or a string (the name of a variable with
//   one of the types cited above, between quotes)
//   * a matrix of strings, each one being the name of a
//   variable
//   * the string 'cte' or 'const' if the user wants a constant
//     to be included automatically
// * grocer_named = a (n x 1) or (1 x n) vector of strings
//   representing the name of variables not entered between
//   quotes
// * grocer_b = a (px1) string vector (of dates) (optional:
//   if not given the function either takes the existing bounds
//   or determines the bounds suitable to the given series)
// * grocer_testna = a boolean indicating whether the program
//   will test the existence of nas values in the matrices of
//   values grocer_y and grocer_x
// * grocer_dropna = a boolean indicating whether the program
//   should keep only lines of matrices grocer_y and grocer_x
//   with non na values in both matrices (suppose that
//   grocer_testna has been set to %f)
// ------------------------------------------------------------
// OUTPUT:
// * grocer_mats = a list of n (T x ki) matrices
// * grocer_names = a list of n (ki x 1) matrices of names
// * grocer_prests = a boolean indicating whether there is a ts
// * grocer_b = a (2*p x 1) string matrix (of dates) or []
// * grocer_nonna = a (T x 1) vector of the indexes with non na values
// ------------------------------------------------------------
// Copyright: Eric Dubois 2006-2009
// http://dubois.ensae.net/grocer.html
 
[grocer_nargout,grocer_nargin] = argn(0)
if grocer_nargin < 6 then
   grocer_mindat=%f
end
 
if grocer_nargin < 5 then
   grocer_dropna=%f
end
 
if grocer_nargin < 4 then
   grocer_testna=%t
end
 
if ~exists('grocer_b','local') then
   // bounds have been entered into the function; they prevail over other existing bounds
   grocer_b=[]
end
 
if isempty(grocer_b) & exists('grocer_boundsvar') then
// bounds have been defined outside the function
   grocer_b=grocer_boundsvar
end
 
[grocer_boundsvarnum]=date2num_m(grocer_b)
grocer_boundsfq=date2fq(grocer_b(1))
 
grocer_n=length(grocer_vars) // number of different types of econometric objects
grocer_mats=list() // the list of grocer_n matrices to be used in the regession
grocer_names=list() // the list of grocer_n vectors of the corresponding variables names
grocer_listtsc=list() // the list of grocer_n ts for each type of econometric objects
grocer_listtsmatc=list() // the list of grocer_n tsmat for each type of econometric objects
grocer_vec0=list() // the list of grocer_n real matrices for each type of econometric objects
grocer_indts0=list() // the list of grocer_n ts indexes for each type of econometric objects
grocer_indtsmat0=list() // the list of grocer_n tsmat indexes for each type of econometric objects
grocer_indvec0=list() // the list of grocer_n real matrices indexes for each type of econometric objects
grocer_indcte0=list() // the list of grocer_n real indexes of specific variables for each type of econometric objects
grocer_n0=list() // the list collecting the number of final variables for each type of econometric objects
grocer_prests=%f
grocer_isvec=%f
grocer_nts=0
 
for grocer_i=1:grocer_n
 
   grocer_li=grocer_vars(grocer_i)
 
// explode the list grocer_i into its various components (real matrices, ts, tsmat, specific variables)
// and their indexes
   [namei,listtsi,grocer_veci,indtsi,indveci,indctei,ni,listtsmati,indtsmati]=...
   explovars(grocer_li,grocer_named(grocer_i))
 
// add the components and indexes to the corresponding list
   grocer_names(grocer_i)=namei
   grocer_listtsc=lstcat(grocer_listtsc,listtsi)
   grocer_listtsmatc=lstcat(grocer_listtsmatc,listtsmati)
   grocer_vec0(grocer_i)=grocer_veci
   grocer_indts0(grocer_i)=indtsi
   grocer_nts=grocer_nts+size(indtsi,'*')
   grocer_indtsmat0(grocer_i)=indtsmati
   grocer_indvec0(grocer_i)=indveci
   grocer_indcte0(grocer_i)=indctei
   grocer_n0(grocer_i)=ni
   grocer_prests = grocer_prests | length(listtsi) ~= 0 | length(listtsmati) ~= 0
   grocer_isvec = grocer_isvec | length(grocer_veci) ~= 0
 
end
 
if grocer_prests then
   if grocer_isvec
      warning('your regression contains ts as well as vectors')
   end
 
   if grocer_b ~= [] then
      grocer_zts=[]
 
      for i=1:size(grocer_listtsc)
          grocer_zts=[grocer_zts ts2vec0(grocer_listtsc(i),grocer_boundsvarnum)]
      end
      for i=1:size(grocer_listtsmatc)
          grocer_zts=[grocer_zts ts2vec0(grocer_listtsmatc(i),grocer_boundsvarnum)]
      end
 
   else
      if grocer_dropna then
         [grocer_zts,grocer_b]=explots(lstcat(grocer_listtsc,grocer_listtsmatc),%f,%t)
      else
         [grocer_zts,grocer_b]=explots(lstcat(grocer_listtsc,grocer_listtsmatc),grocer_testna,grocer_mindat)
      end
 
   end
 
   nobs=size(grocer_zts,1)
   nfirst_ts=1 // nfirst_ts = index of the first ts in the econometric object
   nlast_ts=0    // nlast_ts = index of the last ts in the econometric object
   nfirst_tsmat=grocer_nts+1 // nfirst_tsmat = index of the first ts in the econometric object
   nlast_tsmat=grocer_nts   // nlast_tsmat = index of the last ts in the econometric object
 
   for j=1:grocer_n
   // treat the j-th type of econometric object
      nlast_ts=nlast_ts+size(grocer_indts0(j),1)
      nlast_tsmat=nlast_tsmat+size(grocer_indtsmat0(j),1)
      grocer_yj=ones(nobs,grocer_n0(j)) // grocer_yj = vector of values to fill
      grocer_yj(:,grocer_indts0(j))=grocer_zts(:,nfirst_ts:nlast_ts) // fill the values stemming from ts
      grocer_yj(:,grocer_indtsmat0(j))=grocer_zts(:,nfirst_tsmat:nlast_tsmat) // fill the values stemming from tsmat
      nmatj=size(grocer_vec0(j),1) // number of observations stemming from the real matrices
 
      if nmatj > 0 & nmatj ~= nobs then
         error('nobs of matrices do not match with those of ts')
      end
 
      grocer_yj(:,grocer_indvec0(j))=grocer_vec0(j) // fill the values stemming from real matrices
 
      indctei=grocer_indcte0(j) // index of the specific variables
      namei=grocer_names(j)
      for i=1:size(indctei,1)
         k=indctei(i)
         grocer_yj=deal_varspec(grocer_yj,namei(k),k,grocer_b) // fill the matrix of values for the specific variables
      end
 
      grocer_mats(j)=grocer_yj
      nfirst_ts=nlast_ts+1 // define the index of the first ts for the next econometric object
      nfirst_tsmat=nlast_tsmat+1 // define the index of the first tsmat for the next econometric object
   end
 
 
else
// there is no ts: only real matrices and specifc objects
   fill=[] // indexes of econometric objects that contain explicit values
   for j=1:grocer_n
      if grocer_indvec0(j) ~= [] then
         fill=[fill j]
      end
   end
   empt=1:grocer_n // indexes of econometric objects that do not contain explicit values
   empt(fill)=[]
   nobs=size(grocer_vec0(fill(1)),1)
 
   for i=1:size(fill,2)
      j=fill(i)
      yj=ones(nobs,grocer_n0(j))
      yj(:,grocer_indvec0(j))=grocer_vec0(j) // fill the concerned yj columns with their values
 
      indctei=grocer_indcte0(i)
      namei=grocer_names(i)
      for i=1:size(indctei,1)
         k=indctei(i)
         yj=deal_varspec(yj,namei(k),k,grocer_b) // fill specific variables
      end
 
      indcte=search_cte(yj) // indexes of constant variables
      if size(indcte,2) > 1 then
         warning('you have more than one constant in your '+grocer_named(j)+' set of variables')
         disp('only the first one has been considered')
         yj(:,indcte(2:$))=[]
      end
      grocer_mats(j)=yj
   end
 
   for i=1:size(empt,2)
      j=empt(i)
      indctei=grocer_indcte0(i)
      yj=ones(nobs,grocer_n0(j))
      namei=grocer_names(i)
      for i=1:size(indctei,1)
         j=indctei(i)
         yj=deal_varspec(yj,namei(j),j,grocer_b) // fill specific variables
      end
 
      indcte=search_cte(yj) // indexes of constant variables
      if size(indcte,2) > 1 then
         warning('you have more than one constant in your '+grocer_named(j)+' set of variables')
         disp('only the first one has been considered')
         yj(:,indcte(2:$))=[]
      end
      grocer_mats(j)=yj
   end
   grocer_b=[]
 
end
 
grocer_nonna=%t
mat=[]
for i=1:grocer_n
   mat=[mat grocer_mats(i)]
end
 
grocer_nonna=find(and(~isnan(mat),'c'))
if grocer_dropna then
   for j=1:grocer_n
      matj=grocer_mats(j)
      grocer_mats(j)=matj(grocer_nonna,:)
   end
 
elseif grocer_testna then
 
   for j=1:grocer_n
      presna=find(or(isnan(grocer_mats(j)),'r'))
      namesj=grocer_names(j)
      for i=1:size(presna,2)
          error('series '+namesj(presna(i))+' contains Nan in the range you have specified')
      end
   end
 
end
 
endfunction
 
 
