function [y,namey,prests,grocer_b,grocer_nonna]=explone(grocer_ly,grocer_b,grocer_named,grocer_testna,grocer_dropna,grocer_mindat)
 
// PURPOSE: explode several variables into matrices, names and
// define corresponding bounds
// ------------------------------------------------------------
// INPUT:
// * grocer_ly = list of variables:
//   each element could be
//   - a timeseries, a real vector,
//   a real matrix or a string (the name of a variable with
//   one of the types cited above, between quotes)
//   - a matrix of strings, each one being the name of a
//   variable
//   - the string 'cte' or 'const' if the user wants a constant
//     to be included automatically
// * grocer_b = a (px1) string vector (of dates) (optional:
//   if not given the function either takes the existing bounds
//   or determines the bounds suitable to the given series)
// * grocer_named = a (2x1) vector of strings representing the
//   name of variable not entered between quotes
//   (optional; default = ['endogenous' ; 'exogenous'])
// * grocer_testna = a boolean indicating whether the program
//   will test the existence of na’s values in the matrices of
//   values y
// * grocer_dropna = a boolean indicating whether the program
//   should keep only lines of matrices y and grocer_x
//   with non na values in both matrices (suppose that
//   grocer_testna has been set to %f)
// * grocer_mindat =
//   - %t if the user wants to take the min and max dates
//   over which any series exists
//   - %f if the user wants to take the time period over which
//   all series exist
// ------------------------------------------------------------
// OUTPUT:
// * y = a (T x ky) real vector or a ts
// * namey = a (ky x 1) string vector
// * prests = a boolean indicating whether there is a ts
// * grocer_b = a (2 x 1) string matrix (of dates) or []
// * grocer_nonna = a (T x 1) vector indicating the indexes of
//   non NA observations
// ------------------------------------------------------------
// Copyright: Eric Dubois 2004-2009
// http://dubois.ensae.net/grocer.html
 
 
[grocer_nargout,grocer_nargin] = argn(0)
if grocer_nargin < 6 then
   grocer_mindat=%t
end
if grocer_nargin < 5 then
   grocer_dropna=%t
end
if grocer_nargin < 4 then
   grocer_testna=%t
end
if grocer_nargin < 3 then
   grocer_named='variable'
end
if grocer_nargin < 2 then
   grocer_b = []
end
 
if isempty(grocer_b) & exists('grocer_boundsvar') then
// bounds have been defined outside the function
   grocer_b=grocer_boundsvar
end
 
[grocer_boundsvarnum]=date2num_m(grocer_b)
grocer_boundsfq=date2fq(grocer_b(1))
 
// explode the list of variables into its various components (real matrices, ts, tsmat, specific variables)
// and their indexes
[namey,listtsy,vecy,indtsy,indvecy,indctey,ny,listtsmaty,indtsmaty]=...
   explovars(grocer_ly,grocer_named)
 
prests = (length(listtsy) ~= 0 | length(listtsmaty) ~= 0) | ~isempty(grocer_b)
isvec = (size(vecy,1) ~= 0)
 
if prests then
   if isvec then
      warning('your regression contains ts as well as vectors')
   end
 
   if ~isempty(grocer_b) then
      // create an na series over all the period defined by the bounds
      // there is one application where it is needed: this is in pltseries
      // because it allows to draw series which have not the same time span;
      // in that case explone fills voids with na's...
      tsna=tlist(['ts';'freq';'dates';'series'],grocer_boundsfq,[grocer_boundsvarnum(1):grocer_boundsvarnum($)]',...
      %nan*ones(grocer_boundsvarnum($)-grocer_boundsvarnum(1)+1,1))
      nperiods=size(grocer_boundsvarnum,1)/2
      nobs=sum(grocer_boundsvarnum([1:nperiods]*2)-grocer_boundsvarnum([1:nperiods]*2-1))+nperiods
      zts=[]
      for i=1:size(listtsy)
          zts=[zts ts2vec0(overlay(listtsy(i),tsna),grocer_boundsvarnum)]
      end
      for i=1:size(listtsmaty)
          zts=[zts ts2vec0(listtsmaty(i),grocer_boundsvarnum)]
      end
 
   elseif grocer_dropna then
      [zts,grocer_b]=explots(lstcat(listtsy,listtsmaty),%f,%f)
      nobs=size(zts,1)
 
   else
      [zts,grocer_b]=explots(lstcat(listtsy,listtsmaty),grocer_testna,grocer_mindat)
       nobs=size(zts,1)
 
   end
 
   y=ones(nobs,ny) // y = vector of values to fill
   nts=size(indtsy,'*')
   y(:,indtsy)=zts(:,1:nts) // fill the values stemming from ts
   y(:,indtsmaty)=zts(:,nts+1:$) // fill the values stemming from tsmat
   nmaty=size(vecy,1) // number of observations stemming from the real matrices
 
   if nmaty > 0 & nmaty ~= nobs then
      error('nobs of matrices do not match with those of ts')
   end
 
else
// there is no ts: only real matrices and specifc objects
   if isempty(indvecy) & ~isempty(indctey) then
      error('list of variables cannot contain only specific values, such as const, trend^p')
   end
 
   nobs=size(vecy,1)
   y=ones(nobs,ny)
   grocer_b=[]
 
end
 
y(:,indvecy)=vecy // fill the concerned yj columns with their values
 
for i=1:size(indctey,1)
   k=indctey(i)
   y=deal_varspec(y,namey(k),k,grocer_b) // fill specific variables
end
 
grocer_nonna=find(and(~isnan(y),'c'))
 
if grocer_dropna then
   y=y(grocer_nonna,:)
 
elseif grocer_testna then
 
   presna=find(or(isnan(y),'r'))
   for i=1:size(presna,2)
       error('series '+namey(presna(i))+' contains Nan in the range you have specified')
   end
 
end
 
endfunction
 
